<?php

namespace App\Listeners;

use App\Entity\User;
use App\Response\ApiResponse;
use App\Response\TokenResponse;
use JsonException;
use Lexik\Bundle\JWTAuthenticationBundle\Event\AuthenticationFailureEvent;
use Lexik\Bundle\JWTAuthenticationBundle\Event\AuthenticationSuccessEvent;
use Lexik\Bundle\JWTAuthenticationBundle\Event\JWTExpiredEvent;
use Lexik\Bundle\JWTAuthenticationBundle\Event\JWTInvalidEvent;
use Lexik\Bundle\JWTAuthenticationBundle\Event\JWTNotFoundEvent;
use Symfony\Component\HttpFoundation\Response;

class JwtListener
{
    /**
     * @param AuthenticationSuccessEvent $event
     *
     * @return void
     *
     * @throws JsonException
     */
    public function onAuthenticationSuccessResponse(AuthenticationSuccessEvent $event): void
    {
        $data = $event->getData();
        $user = $event->getUser();

        if (!$user instanceof User) {
            return;
        }

        if ($user->isDeleted()) {
            $response = new ApiResponse();
            $response->addError('Пользователь удален');
        } else {
            $response = new TokenResponse();
            $response->setToken($data['token']);
            $response->addMessage('Здравствуйте, ' . $user->getFullName());
        }

        $data = json_decode($response->getContent(), true, 512, JSON_THROW_ON_ERROR);
        $event->setData($data);
    }

    /**
     * @param AuthenticationFailureEvent $event
     */
    public function onAuthenticationFailureResponse(AuthenticationFailureEvent $event): void
    {
        $response = new ApiResponse();
        $response->addError('Неверный email или пароль');
        $event->setResponse($response);
    }

    /**
     * @param JWTInvalidEvent $event
     */
    public function onJWTInvalid(JWTInvalidEvent $event): void
    {
        $response = new ApiResponse();
        $response->addError('Неверный токен авторизации');
        $response->setStatusCode(Response::HTTP_FORBIDDEN);

        $event->setResponse($response);
    }

    /**
     * @param JWTNotFoundEvent $event
     */
    public function onJWTNotFound(JWTNotFoundEvent $event): void
    {
        $response = new ApiResponse();
        $response->addError('Отсутствует токен');
        $response->setStatusCode(Response::HTTP_FORBIDDEN);

        $event->setResponse($response);
    }

    /**
     * @param JWTExpiredEvent $event
     */
    public function onJWTExpired(JWTExpiredEvent $event): void
    {
        $response = new ApiResponse();
        $response->addError('Срок действия вашего токена истек, пожалуйста, обновите его');
        $response->setStatusCode(Response::HTTP_FORBIDDEN);

        $event->setResponse($response);
    }
}